<?php

declare(strict_types=1);

namespace FiloBlu\Refilo\Helper;

use Magento\Framework\ObjectManagerInterface;
use Magento\PageBuilder\Model\Dom\HtmlDocumentFactory;

use function class_exists;

/**
 *
 */
class PageBuilderConverter
{

    /**
     * @var \Magento\PageBuilder\Model\Dom\HtmlDocumentFactory | null
     */
    private $documentFactory;

    /**
     * @param \Magento\Framework\ObjectManagerInterface $objectManager
     */
    public function __construct(ObjectManagerInterface $objectManager)
    {
        if (class_exists(HtmlDocumentFactory ::class)) {
            $this->documentFactory = $objectManager->get(HtmlDocumentFactory ::class);
        }
    }

    /**
     * @param string|null $data
     * @return string|null
     */
    public function convert($data)
    {
        if (!$this->documentFactory) {
            return $data;
        }

        $document = $this->documentFactory->create(['document' => $data]);
        $body = $document->getContents();
        $styles = $document->querySelectorAll('style');

        $stylesMap = [];

        foreach ($styles as $style) {
            $styleText = $style->getOriginalElement()->nodeValue;
            $styleText = str_replace('#html-body', "\n#html-body", $styleText);
            $outputs = [];
            preg_match_all('/#html\-body\s+?\[data-pb-style\=(.*)\](\{.*\})/m', $styleText, $outputs);
            $stylesMap = array_merge($stylesMap, array_combine($outputs[1], $outputs[2]));
        }

        if (empty($stylesMap)) {
            return $data;
        }

        $toSearch = implode('|', array_keys($stylesMap));
        preg_match('/<body>(.*)<\/body>/s', $body, $matches);

        return preg_replace_callback(
            "/data-pb-style=\"($toSearch)\"/m",
            static function ($matches) use ($stylesMap) {
                return 'style="' . trim($stylesMap[$matches[1]], '{}') . '"';
            },
            $matches[1]
        );
    }
}
